SUBROUTINE read_input
  !OPT
  USE global
  IMPLICIT NONE
  include 'mpif.h'

  ! Local
  INTEGER :: i, itype, n_procs_dummy
  INTEGER :: eof
  DOUBLE PRECISION :: sigrc3
  CHARACTER(LEN=15) :: ensmblstr
  CHARACTER(LEN=10) :: config_type
  CHARACTER(LEN=15),DIMENSION(:),ALLOCATABLE :: configfiles
  CHARACTER(LEN=15) :: molfile  
  CHARACTER(LEN=15) :: option
  CHARACTER(LEN=15) :: trajfile
    

  ! Open the file to read
  OPEN(UNIT=10,FILE='inputfile',STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

    READ(10,*) ensmblstr
    READ(10,*) n_procs_dummy

    ! Initialize MPI if running in parallel
    IF(n_procs_dummy .GT. 1) THEN
      CALL MPI_INIT(mpi_err)
      CALL MPI_COMM_SIZE(MPI_COMM_WORLD,n_procs,mpi_err)
      CALL MPI_COMM_RANK(MPI_COMM_WORLD,my_rank,mpi_err)

      ! Add one to each processor's rank so that it spans [1,n_procs]
      my_rank = my_rank + 1

      ! Throw and error the number of processors isnt consistent
      IF(n_procs_dummy .NE. n_procs)  THEN
        WRITE(*,'(a)') 'THE NUMBER OF PROCESSORS IN THE INPUTFILE DOES NOT MATCH THE NUMBER REQUESTED AT RUNTIME'
        CALL MPI_FINALIZE(mpi_err)
        STOP
      ENDIF

    ! Serial
    ELSE
   
       n_procs = n_procs_dummy
       my_rank = 1

    ENDIF

    ! Initialize the random number generator so it has processor rank information
    CALL seed_random(idum)

    ! Allocate the temperature array and configuration file
    ALLOCATE(temp(n_procs))
    ALLOCATE(configfiles(n_procs))

    ! Read the number of boxes and molecule types
    ! Hardcoded these to reduce complexity of inputfile
    !READ(10,*) n_types, n_sites_max
    n_types = 1
    n_sites_max = 5
  
    ! Set the ensemble type
    IF(ensmblstr .EQ. 'NVT') THEN
      ensmbl = 1
    ELSEIF(ensmblstr .EQ. 'NPT') THEN
      ensmbl = 2
    ELSE
      WRITE(*,*) 'FATAL ERROR: INVALID ENSEMBLE TYPE SPECIFIED IN INPUT FILE'
    ENDIF
   
    IF(my_rank .EQ. 1) THEN
      WRITE(*,'(A,A,I0,A)')TRIM(ensmblstr),' Simulation with ', n_types, ' components'
      WRITE(*,'(A80)')'========================================'&
                     &//'========================================'
    ENDIF


    ! Allocate the box array
    CALL global_allocate('box')
 
    ! Allocate the system arrays
    CALL global_allocate('system')


    ! Read the temperature, pressure 
     READ(10,*) temp(1)
     temp(:) = temp(1)
     IF(ensmbl .EQ. 2) THEN 
       READ(10,*) press
     ENDIF

    ! Read in the information for each molecule type
    DO itype = 1,n_types
      READ(10,*) molfile
      CALL read_molfile(itype,molfile)
    ENDDO
   
    ! Set the potential parameters
    CALL set_param('potential')
  
      ! Read the integration time step, # MD steps per move and max log volume
      READ(10,*) tstep, n_md_steps, max_vol

      ! Read files names for restart configuration from file
      OPEN(UNIT=11,FILE='configs',STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

        ! Input files
        DO i_proc = 1,n_procs
          READ(11,*) configfiles(i_proc)
          IF(my_rank .EQ. 1) WRITE(*,'(A,I0,2A)') 'Restarting simulation for replica ',i_proc,' from congifuration file ', TRIM(configfiles(i_proc))
        ENDDO

      CLOSE(UNIT=11)

      ! Read in the configuration file
      CALL set_initconf(configfiles(my_rank))

    ! Set the system parameters
    CALL set_param('system')


    ! Read the run statistics
    READ(10,*) n_blocks,n_equil,n_steps

    ! Allocate the block average arrays
    CALL global_allocate('block')


    ! Set the end of file flag
    eof = 0

    ! Continue reading options until end of file
    DO 
     
      ! Read the option
      READ(10,*,IOSTAT=eof) option
   
      ! Check for unspecified errors
      IF(eof > 0 ) THEN
        WRITE(*,'(A)') 'FATAL ERROR: I/O ERROR IN READING THE  INPUTFILE'
        STOP
      ENDIF

      ! Exit if it is the end of the file
      IF(eof < 0 ) EXIT

      ! Dump trajectory to file
      IF(option .EQ. 'dump_trajxyz') THEN

        dump_traj = 1

        ! Read the number of MC steps between trajectory dumps
        READ(10,*) n_traj


      ! Ewald summation for charges systems
      ELSEIF(option .EQ. 'ewald_vacuum') THEN

        ! Set ewald flag (ON)
        ewld = 1
        ewld_vac = 1

        ! Read the error tolerance
        READ(10,*) etol


        ewld_box(:) = box(:)
       
        ! Set parameters and allocate the arrays
        CALL set_ewld(0,0)

        ! Setup the k-vectors for each box
        CALL set_ewld(1)
    
     ELSEIF(option .EQ. 'ljlrc') THEN

       sigrc3 = (sig(1,1,1,1)/rmax(1,1))**3
       lrc_fact_npt = (8.0d0/3.0d0)*Pi*eps(1,1,1,1)*(sig(1,1,1,1)**3)*(n_mol_tot**2)
       lrc_fact_npt = -lrc_fact_npt*sigrc3
       IF(my_rank .EQ. 1) WRITE(*,*) 'LONG-RANGE DISPERSION CORECTION FACTOR FOR VOLUME MOVES SET'

     ! Configurational bias, weighted ensemble 
     ELSEIF(option .EQ. 'weighted') THEN

        ! Set the umbrella sampling falg (ON)
        wensmbl = 1

        ! Read the number of CVs and the frequency (in steps) for dumping CVs
        n_cvs = 2  ! Hardcode the number of CVs
        READ(10,*) n_cvdump

        ! Initialize arrays and read CV input
        CALL weighted_ensmbl(0,0,0,0,0)

     ! End of file
     ELSEIF((option .EQ. 'eof') .OR. (option .EQ. 'Eof') .OR. (option .EQ. 'EOF')) THEN

        EXIT


      ! Throw an error for unknown option
      ELSE

        WRITE(*,'(3A)') 'FATAL ERROR: UNEXPECTED OPTION ',TRIM(option),' AT THE END OF THE INPUTFILE'
        STOP

      ENDIF

    ENDDO


    ! Allocate the arrays for some optional statistical variables
    CALL global_allocate('optional')

    ! Use ST2 with RF
    IF(ewld .NE. 1) THEN

      ! Interaction cutoffs
      rmax(:,:) = 7.80d0
      rmaxsq(:,:) = 7.80d0**2

      sigrc3 = (sig(1,1,1,1)/rmax(1,1))**3
      lrc_fact_npt =(8.0d0/3.0d0)*Pi*eps(1,1,1,1)*(sig(1,1,1,1)**3)*(n_mol_tot**2)
      lrc_fact_npt = -lrc_fact_npt*sigrc3

      IF(my_rank .EQ. 1) WRITE(*,*)"EWALD NOT SELECTED, USING REACTION FIELD WITH 7.8 A CUTOFF & DISPERSION LRC"

    ENDIF


  CLOSE(UNIT=10)


  RETURN

END SUBROUTINE read_input
