SUBROUTINE read_molfile(itype,filename)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: itype
  CHARACTER(LEN=*) :: filename

  ! Local
  INTEGER :: isite,jsite
  INTEGER :: ierr, n_inrta_zero
  DOUBLE PRECISION :: net_charge
  DOUBLE PRECISION :: rxc,ryc,rzc
  CHARACTER(LEN=15) :: temp_name

  ! Initialize the number of dispersion sites and ion sites
  n_disp(itype) = 0
  n_ions(itype) = 0
  disp_sites(:,itype) = 0
  ion_sites(:,itype) = 0
 
  ! Initialize the net charge
  net_charge = 0.0d0

  ! Open the file to read
  OPEN(UNIT=15,FILE=filename,STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

    ! Read the name of the molecule
    READ(15,*) temp_name
       
      ! Check that name length
      IF(LEN(TRIM(temp_name)) .GT. 4) THEN
        WRITE(*,'(5A)') 'FATAL ERROR: MOLECULE ',TRIM(temp_name),' IN FILE ', TRIM(filename),' IS MORE THAN 4 CHARACTERS'  
        STOP
      ENDIF    

      ! Set the name of the molecule
      mol_name(itype) = TRIM(temp_name)

    ! Read the mass of the molecule (g/mol)
    READ(15,*) mol_mass(itype)
     
    ! Read the potential cutoff distance for the molecule
    READ(15,*) rmax(itype,itype)

    ! Read the number of sites on the molecule
    READ(15,*) n_sites(itype)
   
      ! Check the number of sites against to global maximum
      IF(n_sites(itype) .GT. n_sites_max) THEN
        WRITE(*,'(3A)') 'FATAL ERROR: MOLECULE ', TRIM(mol_name(itype)), ' HAS MORE THAN THE MAXIMUM NUMBER OF SITES SET IN THE INPUTFILE' 
        STOP
      ENDIF

    ! Read the information for each molecular site
    DO isite = 1,n_sites(itype)
   
      ! Read the name of the site
      READ(15,*) temp_name

        ! Check that name length
        IF(LEN(TRIM(temp_name)) .GT. 4) THEN
          WRITE(*,'(5A)') 'FATAL ERROR: SITE ',TRIM(temp_name),' ON MOLECULE ', TRIM(mol_name(itype)),' IS MORE THAN 4 CHARACTERS'
          STOP
        ENDIF
 
        ! Set the name of the site
        site_name(isite,itype) = TRIM(temp_name)

      READ(15,*) site_mass(isite,itype)
      READ(15,*) rx_i(isite,itype),ry_i(isite,itype),rz_i(isite,itype)
      READ(15,*) sig(isite,isite,itype,itype),eps(isite,isite,itype,itype),&
                 &alpha(isite,isite,itype,itype),q(isite,isite,itype,itype)
    ENDDO

  
  CLOSE(UNIT=15)


  ! Check molecule geometry for errors

  ! Compute the molecule's mass
  mol_mass(itype) = SUM(site_mass(:,itype))

  ! Check that the center of mass is at (0,0,0) 
  rxc = SUM(site_mass(:,itype)*rx_i(:,itype))/mol_mass(itype)
  ryc = SUM(site_mass(:,itype)*ry_i(:,itype))/mol_mass(itype)
  rzc = SUM(site_mass(:,itype)*rz_i(:,itype))/mol_mass(itype)

  IF((ABS(rxc) .GT. 1.0d-6) .OR. (ABS(ryc) .GT. 1.0d-6) .OR. (ABS(rzc) .GT. 1.0d-6)) THEN
    WRITE(*,'(3A)') 'FATAL ERROR: CENTER OF MASS OF ON MOLECULE', TRIM(mol_name(itype)),&
                        &' NOT AT 0,0,0'
    WRITE(*,*) rxc,ryc,rzc
    STOP
  ELSE
   rx_i(:,itype) = rx_i(:,itype) - rxc
   ry_i(:,itype) = ry_i(:,itype) - ryc
   rz_i(:,itype) = rz_i(:,itype) - rzc
  ENDIF
 

  ! Compute the inertia tensor and check that it is diagonal
  inrta_tnsr(1,1,itype) = SUM(site_mass(:,itype)*(ry_i(:,itype)**2 + rz_i(:,itype)**2))
  inrta_tnsr(2,2,itype) = SUM(site_mass(:,itype)*(rx_i(:,itype)**2 + rz_i(:,itype)**2))
  inrta_tnsr(3,3,itype) = SUM(site_mass(:,itype)*(rx_i(:,itype)**2 + ry_i(:,itype)**2))
  inrta_tnsr(1,2,itype) = SUM(-site_mass(:,itype)*rx_i(:,itype)*ry_i(:,itype))
  inrta_tnsr(1,3,itype) = SUM(-site_mass(:,itype)*rx_i(:,itype)*rz_i(:,itype))
  inrta_tnsr(2,3,itype) = SUM(-site_mass(:,itype)*ry_i(:,itype)*rz_i(:,itype))

  IF((ABS(inrta_tnsr(1,2,itype)) .GT. 1.0d-8) .OR. (ABS(inrta_tnsr(1,3,itype)) .GT. 1.0d-8) .OR. (ABS(inrta_tnsr(2,3,itype)) .GT. 1.0d-8)) THEN
    WRITE(*,'(3A)') 'FATAL ERROR: INERTIA TENSOR OF MOLECULE ', TRIM(mol_name(itype)),&
                        &' NOT DIAGONAL'
    STOP

  ENDIF
  
  ! Make sure the the Ixx > Iyy > Izz
  IF((inrta_tnsr(1,1,itype) .LT. inrta_tnsr(2,2,itype)) .OR.(inrta_tnsr(2,2,itype) .LT. inrta_tnsr(3,3,itype))) THEN
        WRITE(*,'(3A)') 'FATAL ERROR: INERTIA TENSOR OF MOLECULE ',TRIM(mol_name(itype)),&
                        &' NOT IN ORDER Ixx > Iyy > Izz'
  ENDIF

  ! Compute the inverse
  n_inrta_zero = 0

  ! Ixx
  IF(ABS(inrta_tnsr(1,1,itype)) .LT. 1.0d-7) THEN
    n_inrta_zero = n_inrta_zero + 1
    inrta_tnsr_inv(1,1,itype) = 0.0d0
  ELSE
    inrta_tnsr_inv(1,1,itype) = 1.0d0/inrta_tnsr(1,1,itype)
  ENDIF

  ! Iyy
  IF(ABS(inrta_tnsr(2,2,itype)) .LT. 1.0d-7) THEN
    n_inrta_zero = n_inrta_zero + 1
    inrta_tnsr_inv(2,2,itype) = 0.0d0
  ELSE
    inrta_tnsr_inv(2,2,itype) = 1.0d0/inrta_tnsr(2,2,itype)
  ENDIF

  ! Izz
  IF(ABS(inrta_tnsr(3,3,itype)) .LT. 1.0d-7) THEN
    n_inrta_zero = n_inrta_zero + 1
    inrta_tnsr_inv(3,3,itype) = 0.0d0
  ELSE
    inrta_tnsr_inv(3,3,itype) = 1.0d0/inrta_tnsr(3,3,itype)
  ENDIF


  ! Compute the number of rotational degrees of freedom
  IF(n_inrta_zero .EQ. 3) THEN 
    n_rot_dof(itype) = 0                  ! Spherical
  ELSEIF(n_inrta_zero .EQ. 1) THEN
    n_rot_dof(itype) = 2                  ! Linear
  ELSE
    n_rot_dof(itype) = 3                  ! 3 rotational axes  
  ENDIF

  WRITE(*,*) 'INERTIAL TENSOR:'
  WRITE(*,'(3F28.8,I5)') inrta_tnsr(1,1,itype), inrta_tnsr(2,2,itype),inrta_tnsr(3,3,itype), n_rot_dof(itype)


  ! Error checking

  ! Loop over the sites
  DO isite = 1,n_sites(itype)

    ! Check for consistency of dispersion parameters
    IF((sig(isite,isite,itype,itype) .EQ. 0.0d0) .OR. (eps(isite,isite,itype,itype) .EQ. 0.0d0)) THEN

      ! Initialize the error handler
      ierr = 0

      ! Sig, eps and alpha should be zero
      IF(sig(isite,isite,itype,itype) .NE. 0.0d0) ierr = 1
      IF(eps(isite,isite,itype,itype) .NE. 0.0d0) ierr = 1
      IF(alpha(isite,isite,itype,itype) .NE. 0.0d0) ierr = 1

      ! Check for consistency
      IF(ierr .NE. 0) THEN
        WRITE(*,'(5A)') 'FATAL ERROR: ALL DISPERIONS PARAMETERS (SIG,EPS,ALPHA) ON SITE ',&
                        &TRIM(site_name(isite,itype)), ' ON MOLECULE ', TRIM(mol_name(itype)),&
                        &' MUST BE ZERO' 
        STOP
      ENDIF
    ENDIF


    ! Check for mislabeled sites

    ! Loop over the previous sites
    DO jsite = 1,isite-1

      ! Initialize the error handler
      ierr = 0

      ! If they do not have the same name it might be a new site type
      IF(site_name(jsite,itype) .NE. site_name(isite,itype)) THEN
          
        ! Check that the sites DONT have the same parameters
        IF(sig(jsite,jsite,itype,itype) .EQ. sig(isite,isite,itype,itype)) ierr = ierr +  1
        IF(eps(jsite,jsite,itype,itype) .EQ. eps(isite,isite,itype,itype)) ierr = ierr + 1
        IF(alpha(jsite,jsite,itype,itype) .EQ. alpha(isite,isite,itype,itype)) ierr = ierr + 1
        IF(q(jsite,jsite,itype,itype) .EQ. q(isite,isite,itype,itype)) ierr = ierr + 1

        ! Check for a mismatch
        IF(ierr .EQ. 5) THEN
          WRITE(*,'(3A)') 'FATAL ERROR: TWO SITES ON ',TRIM(mol_name(itype)), ' HAVE THE SAME PARAMETERS BUT DIFFERENT NAMES'
          STOP
        ENDIF

      ELSE
          
        ! Initialize the error handler 
        ierr = 0

        ! Check that the sites have the same parameters
        IF(sig(jsite,jsite,itype,itype) .NE. sig(isite,isite,itype,itype)) ierr = 1
        IF(eps(jsite,jsite,itype,itype) .NE. eps(isite,isite,itype,itype)) ierr = 1
        IF(alpha(jsite,jsite,itype,itype) .NE. alpha(isite,isite,itype,itype)) ierr = 1
        IF(q(jsite,jsite,itype,itype) .NE. q(isite,isite,itype,itype)) ierr = 1

        ! Check for a mismatch
        IF(ierr .NE. 0) THEN
          WRITE(*,'(3A)') 'FATAL ERROR: TWO SITES ON ',TRIM(mol_name(itype)), ' HAVE THE SAME NAME BUT DIFFERENT PARAMETERS'
          STOP
        ENDIF

      ENDIF
    ENDDO


    ! Update the dispersion site list
    IF(sig(isite,isite,itype,itype) .NE. 0.0) THEN
   
      ! Update the number of dispersion sites on itype
      n_disp(itype) = n_disp(itype) + 1

      ! Update the dispersion site list for itype
      disp_sites(n_disp(itype),itype) = isite

    ENDIF 

    ! Update the ion list
    IF(q(isite,isite,itype,itype) .NE. 0.0d0) THEN
 
      ! Update the number of ions on itype
      n_ions(itype) = n_ions(itype) + 1
 
      ! Update the ion list for itype
      ion_sites(n_ions(itype),itype) = isite 

      ! Update the net charge
      net_charge = net_charge + q(isite,isite,itype,itype)

    ENDIF

  ! End loop over the sites
  ENDDO


  ! Throw a warning if the molecule has a net charge
  IF(DABS(net_charge) .GT. 1.0d-8) WRITE(*,'(3A,F10.5,A)') 'WARNING: ', TRIM(mol_name(itype)), ' HAS A NET CHARGE OF ', net_charge, ' e'
  

  RETURN

END SUBROUTINE read_molfile
