SUBROUTINE weighted_ensmbl(selector,move_type,imol,cv_data,cv_eng_chng)

  USE global
  IMPLICIT NONE
  INCLUDE 'mpif.h'

  ! Passed
  INTEGER :: selector
  INTEGER:: move_type,imol
  DOUBLE PRECISION,DIMENSION(3) :: cv_data
  DOUBLE PRECISION :: cv_eng_chng

  
  ! Local 
  
  ! Looping indices
  INTEGER :: icv,iparam,icvtype,isim

  ! Collective variable information  
  CHARACTER(LEN=15) :: selcv

  ! Error handler
  INTEGER :: ierr


  ! Return if weighted ensemble is not on
  IF(wensmbl .NE. 1) RETURN

 
  SELECT CASE (selector)


    ! Read collective variable (cv) information from inputfile (UNIT=10) and preform initializations
    CASE(0)
      
      ! Allocate collective variable arrays
      IF(.NOT. ALLOCATED(cv_types))   ALLOCATE(cv_types(n_cvs), STAT=ierr)
      IF(.NOT. ALLOCATED(cv_moves))   ALLOCATE(cv_moves(4,6),  STAT=ierr)
      IF(.NOT. ALLOCATED(cv_eng))     ALLOCATE(cv_eng(n_cvs), STAT=ierr)
      IF(.NOT. ALLOCATED(cv_eng_new)) ALLOCATE(cv_eng_new(n_cvs), STAT=ierr)
      IF(.NOT. ALLOCATED(rho))        ALLOCATE(rho(n_procs),  STAT=ierr)
      IF(.NOT. ALLOCATED(rhostar))    ALLOCATE(rhostar(n_procs),  STAT=ierr)
      IF(.NOT. ALLOCATED(krho))       ALLOCATE(krho(n_procs),  STAT=ierr)
      IF(.NOT. ALLOCATED(Q6))         ALLOCATE(Q6(n_procs),  STAT=ierr)
      IF(.NOT. ALLOCATED(Q6star))     ALLOCATE(Q6star(n_procs),  STAT=ierr)
      IF(.NOT. ALLOCATED(kQ6))        ALLOCATE(kQ6(n_procs),  STAT=ierr)

      ! Check for allocation error
      IF(ierr .NE. 0) THEN
        WRITE(*,*) 'FATA ERROR: ALLOCATION OF THE EGRID ARRAY FAILED IN WEIGHTED ENSEMBLE SUBROUTINE'
        STOP
      ENDIF
 

      ! Initialize values
      cv_moves = 0 
      cv_eng = 0.0d0
      cv_eng_new = 0.0d0
      krho = 0.0d0
      rhostar = 0.0d0
      rho = 0.0d0
      kQ6 = 0.0d0
      Q6star = 0.0d0
      Q6 = 0.0d0


      ! Specify cv_move array (0=off, 1 = on)
      ! Index 1, move type: 1 = displacements; 2 = rotations; 3 = volume changes; 4 = particle deletions/insertions (including transfers)
      ! Index 2, 1 = density; 2 = Steinhardt Q6

      ! Density --> volume displacements and particle deletions/insertions
      cv_moves(3,1) = 1
      cv_moves(4,1) = 1

      ! Steinhardt's Q6 --> displacements and particle deletions/insertions
      cv_moves(1,2) = 1
      cv_moves(4,2) = 1



      ! Set the CV number
      icv = 1

      ! Read in rho information
      OPEN(UNIT=11,FILE='rho',STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

        ! Set CV type
        cv_types(icv) = 1

        ! Read the input parameters
        ! iparam value: 1 = spring constant (units kT), 2 = lower density bound,
        ! 3 = upper density bound, 4 = gaussian sigma
        DO i_proc = 1,n_procs
          READ(11,*) krho(i_proc), rhostar(i_proc)
        ENDDO

      CLOSE(UNIT=11)


      ! Set the CV number
      icv = 2

      ! Read in Q6 information
      OPEN(UNIT=11,FILE='Q6',STATUS='OLD',ACCESS='SEQUENTIAL',ACTION='READ')

        ! Set CV type
        cv_types(icv) = 2

        ! Read the input parameters
        DO i_proc = 1,n_procs
          READ(11,*) kQ6(my_rank),  Q6star(my_rank)
        ENDDO

      CLOSE(UNIT=11)

    ! Calculate the CVs and energies from scratch (initialize)
    CASE(1)
 
      ! Calculate the value of each CV

      ! Loop over the CVs
      DO icv = 1,n_cvs 

        ! Calculate icv's value
        CALL cv_total(icv)

      ENDDO


      ! Calculate the biasing energy for each CV (in Kelvin)

      ! Loop over the CVs
      DO icv = 1,n_cvs

        ! Density
        IF(cv_types(icv) .EQ. 1) THEN

          cv_eng(icv) = -0.5d0*temp(my_rank)*krho(my_rank)*(rho(my_rank) - rhostar(my_rank))**2

        ! Q6
        ELSEIF(cv_types(icv)  .EQ. 2) THEN
      
          cv_eng(icv) =-0.5d0*temp(my_rank)*kQ6(my_rank)*(Q6(my_rank) - Q6star(my_rank))**2
  
        ENDIF

      ENDDO


    ! Calculate the new CVs, new energies and the relative weight 
    CASE(2)

      ! Initialize the weight and energy change
      cv_eng_chng = 0.0d0

      ! Initialize the new CV and CV energies to old values so
      ! change will be 0 if they are not affected by the move
      rho_new = rho(my_rank)
      Q6_new  = Q6(my_rank)
      cv_eng_new = cv_eng


      ! Calculate the value of each CV

      ! Loop over the CVs
      DO icv = 1,n_cvs

        ! Cycle if the CV is not updated for the MC move type
        IF(cv_moves(move_type,cv_types(icv)) .NE. 1) CYCLE

        ! Update the CV
        CALL cv(icv,imol,cv_data)

      ENDDO


      ! Calculate the biasing energy for each CV (in Kelvin)

      ! Loop over the CVs
      DO icv = 1,n_cvs

        ! Calculate the biasing energy for each CV (in Kelvin)

        ! Density
        IF(cv_types(icv) .EQ. 1) THEN

          cv_eng_new(icv) = -0.5d0*temp(my_rank)*krho(my_rank)*(rho_new - rhostar(my_rank))**2

        ! Q6
        ELSEIF(cv_types(icv)  .EQ. 2) THEN

          cv_eng_new(icv) = -0.5d0*temp(my_rank)*kQ6(my_rank)*(Q6_new - Q6star(my_rank))**2

        ENDIF


        ! Calculate the change in energy (take negative of the change since we are adding
        ! it to the potential energy before taking the exponent and accepting/rejecting)
        cv_eng_chng = cv_eng_chng - (cv_eng_new(icv) - cv_eng(icv))
      
      ENDDO


    ! Update the CVs and energies if the move is accepted
    CASE(3)

      ! Loop over the CVs    
      DO icv = 1,n_cvs

        ! Cycle if the CV is not updated for the MC move type
        IF(cv_moves(move_type,cv_types(icv)) .NE. 1) CYCLE

        ! Density
        IF(cv_types(icv) .EQ. 1) THEN
      
          rho(my_rank) = rho_new

        ! Q6
        ELSEIF(cv_types(icv)  .EQ. 2) THEN

          Q6(my_rank) = Q6_new

        ENDIF

        ! Update the CV energy
        cv_eng(icv) = cv_eng_new(icv)

      ENDDO


    CASE DEFAULT

      WRITE(*,*) 'FATAL ERROR: INVALID FLAG TYPE IN WEIGHTED_ENSEMBLE SUBROUTINE'
      STOP

  END SELECT


  RETURN

END SUBROUTINE weighted_ensmbl
